@@ -15,7 +15,7 @@ namespace phpbb\viglink\migrations;
  */
 class viglink_data extends \phpbb\db\migration\migration
 {
-	static public function depends_on()
+	public static function depends_on()
 	{
 		return array('\phpbb\db\migration\data\v31x\v312');
 	}
