@@ -23,7 +23,7 @@ class local extends \phpbb\avatar\driver\driver
 	*/
 	public function get_data($row)
 	{
-		$root_path = (defined('PHPBB_USE_BOARD_URL_PATH') && PHPBB_USE_BOARD_URL_PATH) ? generate_board_url() . '/' : $this->path_helper->get_web_root_path();
+		$root_path = $this->path_helper->get_web_root_path();
 
 		return array(
 			'src' => $root_path . $this->config['avatar_gallery_path'] . '/' . $row['avatar'],
@@ -64,7 +64,7 @@ class local extends \phpbb\avatar\driver\driver
 
 			$table_cols = isset($row['avatar_gallery_cols']) ? $row['avatar_gallery_cols'] : 4;
 			$row_count = $col_count = $avatar_pos = 0;
-			$avatar_count = sizeof($avatar_list[$category]);
+			$avatar_count = count($avatar_list[$category]);
 
 			reset($avatar_list[$category]);
 
@@ -84,11 +84,13 @@ class local extends \phpbb\avatar\driver\driver
 					'AVATAR_IMAGE'  => $this->phpbb_root_path . $this->config['avatar_gallery_path'] . '/' . $img['file'],
 					'AVATAR_NAME' 	=> $img['name'],
 					'AVATAR_FILE' 	=> $img['filename'],
+					'CHECKED'			=> $img['file'] === $row['avatar'],
 				));
 
 				$template->assign_block_vars('avatar_local_row.avatar_local_option', array(
 					'AVATAR_FILE' 		=> $img['filename'],
-					'S_OPTIONS_AVATAR'	=> $img['filename']
+					'S_OPTIONS_AVATAR'	=> $img['filename'],
+					'CHECKED'			=> $img['file'] === $row['avatar'],
 				));
 
 				$col_count = ($col_count + 1) % $table_cols;
@@ -156,7 +158,7 @@ class local extends \phpbb\avatar\driver\driver
 	*/
 	protected function get_avatar_list($user)
 	{
-		$avatar_list = ($this->cache == null) ? false : $this->cache->get('_avatar_local_list');
+		$avatar_list = ($this->cache == null) ? false : $this->cache->get('_avatar_local_list_' . $user->data['user_lang']);
 
 		if ($avatar_list === false)
 		{
@@ -172,17 +174,19 @@ class local extends \phpbb\avatar\driver\driver
 				// Match all images in the gallery folder
 				if (preg_match('#^[^&\'"<>]+\.(?:' . implode('|', $this->allowed_extensions) . ')$#i', $image) && is_file($file_path . '/' . $image))
 				{
-					if (function_exists('getimagesize'))
-					{
-						$dims = getimagesize($file_path . '/' . $image);
-					}
-					else
+					$dims = $this->imagesize->getImageSize($file_path . '/' . $image);
+
+					if ($dims === false)
 					{
 						$dims = array(0, 0);
 					}
+					else
+					{
+						$dims = array($dims['width'], $dims['height']);
+					}
 					$cat = ($path == $file_path) ? $user->lang['NO_AVATAR_CATEGORY'] : str_replace("$path/", '', $file_path);
 					$avatar_list[$cat][$image] = array(
-						'file'      => ($cat != $user->lang['NO_AVATAR_CATEGORY']) ? rawurlencode($cat) . '/' . rawurlencode($image) : rawurlencode($image),
+						'file'      => ($cat != $user->lang['NO_AVATAR_CATEGORY']) ? str_replace('%2F', '/', rawurlencode($cat)) . '/' . rawurlencode($image) : rawurlencode($image),
 						'filename'  => rawurlencode($image),
 						'name'      => ucfirst(str_replace('_', ' ', preg_replace('#^(.*)\..*$#', '\1', $image))),
 						'width'     => $dims[0],
@@ -194,7 +198,7 @@ class local extends \phpbb\avatar\driver\driver
 
 			if ($this->cache != null)
 			{
-				$this->cache->put('_avatar_local_list', $avatar_list, 86400);
+				$this->cache->put('_avatar_local_list_' . $user->data['user_lang'], $avatar_list, 86400);
 			}
 		}
 
