@@ -15,6 +15,7 @@ namespace phpbb\console\command\extension;
 use Symfony\Component\Console\Input\InputArgument;
 use Symfony\Component\Console\Input\InputInterface;
 use Symfony\Component\Console\Output\OutputInterface;
+use Symfony\Component\Console\Style\SymfonyStyle;
 
 class purge extends command
 {
@@ -33,19 +34,22 @@ class purge extends command
 
 	protected function execute(InputInterface $input, OutputInterface $output)
 	{
+		$io = new SymfonyStyle($input, $output);
+
 		$name = $input->getArgument('extension-name');
 		$this->manager->purge($name);
 		$this->manager->load_extensions();
 
 		if ($this->manager->is_enabled($name))
 		{
-			$output->writeln('<error>' . $this->user->lang('CLI_EXTENSION_PURGE_FAILURE', $name) . '</error>');
+			$io->error($this->user->lang('CLI_EXTENSION_PURGE_FAILURE', $name));
 			return 1;
 		}
 		else
 		{
 			$this->log->add('admin', ANONYMOUS, '', 'LOG_EXT_PURGE', time(), array($name));
-			$output->writeln('<info>' . $this->user->lang('CLI_EXTENSION_PURGE_SUCCESS', $name) . '</info>');
+			$this->check_apcu_cache($io);
+			$io->success($this->user->lang('CLI_EXTENSION_PURGE_SUCCESS', $name));
 			return 0;
 		}
 	}
