@@ -59,6 +59,11 @@ class manager
 	 */
 	protected $php_ext;
 
+	/**
+	 * @var \phpbb\template\template
+	 */
+	protected $template;
+
 	/**
 	* Constructor. Loads all available tasks.
 	*
@@ -66,13 +71,15 @@ class manager
 	* @param helper $routing_helper Routing helper
 	* @param string $phpbb_root_path Relative path to phpBB root
 	* @param string $php_ext PHP file extension
+	* @param \phpbb\template\template $template
 	*/
-	public function __construct(ContainerInterface $phpbb_container, helper $routing_helper, $phpbb_root_path, $php_ext)
+	public function __construct(ContainerInterface $phpbb_container, helper $routing_helper, $phpbb_root_path, $php_ext, $template)
 	{
 		$this->phpbb_container = $phpbb_container;
 		$this->routing_helper = $routing_helper;
 		$this->phpbb_root_path = $phpbb_root_path;
 		$this->php_ext = $php_ext;
+		$this->template = $template;
 	}
 
 	/**
@@ -80,8 +87,6 @@ class manager
 	* and puts them into $this->tasks.
 	*
 	* @param array|\Traversable $tasks		Array of instances of \phpbb\cron\task\task
-	*
-	* @return null
 	*/
 	public function load_tasks($tasks)
 	{
@@ -94,8 +99,6 @@ class manager
 	/**
 	* Loads registered tasks from the container, wraps them
 	* and puts them into $this->tasks.
-	*
-	* @return null
 	*/
 	public function load_tasks_from_container()
 	{
@@ -116,7 +119,7 @@ class manager
 	*
 	* If no tasks are ready, null is returned.
 	*
-	* @return \phpbb\cron\task\wrapper|null
+	* @return wrapper|null
 	*/
 	public function find_one_ready_task()
 	{
@@ -161,7 +164,7 @@ class manager
 	* Web runner uses this method to resolve names to tasks.
 	*
 	* @param string				$name Name of the task to look up.
-	* @return \phpbb\cron\task\wrapper	A wrapped task corresponding to the given name, or null.
+	* @return wrapper	A wrapped task corresponding to the given name, or null.
 	*/
 	public function find_task($name)
 	{
@@ -193,10 +196,10 @@ class manager
 	* Wraps a task inside an instance of \phpbb\cron\task\wrapper.
 	*
 	* @param  \phpbb\cron\task\task 			$task The task.
-	* @return \phpbb\cron\task\wrapper	The wrapped task.
+	* @return wrapper	The wrapped task.
 	*/
 	public function wrap_task(\phpbb\cron\task\task $task)
 	{
-		return new wrapper($task, $this->routing_helper, $this->phpbb_root_path, $this->php_ext);
+		return new wrapper($task, $this->routing_helper, $this->phpbb_root_path, $this->php_ext, $this->template);
 	}
 }
