@@ -60,6 +60,14 @@ abstract class mssql_base extends \phpbb\db\driver\driver
 		return $expression . " ESCAPE '\\'";
 	}
 
+	/**
+	* {@inheritDoc}
+	*/
+	function cast_expr_to_bigint($expression)
+	{
+		return 'CONVERT(BIGINT, ' . $expression . ')';
+	}
+
 	/**
 	* Build db-specific query data
 	* @access private
@@ -68,4 +76,12 @@ abstract class mssql_base extends \phpbb\db\driver\driver
 	{
 		return $data;
 	}
+
+	/**
+	* {@inheritDoc}
+	*/
+	function sql_quote($msg)
+	{
+		return '"' . $msg . '"';
+	}
 }
