@@ -17,12 +17,12 @@ class v320a1 extends \phpbb\db\migration\container_aware_migration
 {
 	public function effectively_installed()
 	{
-		return version_compare($this->config['version'], '3.2.0-a1', '>=');
+		return phpbb_version_compare($this->config['version'], '3.2.0-a1', '>=');
 	}
 
 	static public function depends_on()
 	{
-		return array(
+		return [
 			'\phpbb\db\migration\data\v320\dev',
 			'\phpbb\db\migration\data\v320\allowed_schemes_links',
 			'\phpbb\db\migration\data\v320\announce_global_permission',
@@ -32,13 +32,13 @@ class v320a1 extends \phpbb\db\migration\container_aware_migration
 			'\phpbb\db\migration\data\v320\log_post_id',
 			'\phpbb\db\migration\data\v320\remove_outdated_media',
 			'\phpbb\db\migration\data\v320\notifications_board',
-		);
+		];
 	}
 
 	public function update_data()
 	{
-		return array(
-			array('config.update', array('version', '3.2.0-dev')),
-		);
+		return [
+			['config.update', ['version', '3.2.0-a1']],
+		];
 	}
 }
