@@ -339,9 +339,10 @@ class migrator
 			$depend = $this->get_valid_name($depend);
 
 			// Test all possible namings before throwing exception
-			if ($this->unfulfillable($depend) !== false)
+			$missing = $this->unfulfillable($depend);
+			if ($missing !== false)
 			{
-				throw new \phpbb\db\migration\exception('MIGRATION_NOT_FULFILLABLE', $name, $depend);
+				throw new \phpbb\db\migration\exception('MIGRATION_NOT_FULFILLABLE', $name, $missing);
 			}
 
 			if (!isset($this->migration_state[$depend]) ||
