@@ -13,10 +13,11 @@
 
 namespace phpbb\event;
 
-use Symfony\Component\EventDispatcher\ContainerAwareEventDispatcher;
+use Symfony\Component\EventDispatcher\EventDispatcher;
+use Symfony\Component\EventDispatcher\Event;
 
 /**
-* Extension of the Symfony2 EventDispatcher
+* Extension of the Symfony EventDispatcher
 *
 * It provides an additional `trigger_event` method, which
 * gives some syntactic sugar for dispatching events. Instead
@@ -29,15 +30,54 @@ use Symfony\Component\EventDispatcher\ContainerAwareEventDispatcher;
 *     extract($phpbb_dispatcher->trigger_event('core.index', compact($vars)));
 *
 */
-class dispatcher extends ContainerAwareEventDispatcher implements dispatcher_interface
+class dispatcher extends EventDispatcher implements dispatcher_interface
 {
+	/**
+	 * @var bool
+	 */
+	protected $disabled = false;
+
 	/**
 	* {@inheritdoc}
 	*/
-	public function trigger_event($eventName, $data = array())
+	public function trigger_event($eventName, $data = [])
 	{
 		$event = new \phpbb\event\data($data);
 		$this->dispatch($eventName, $event);
 		return $event->get_data_filtered(array_keys($data));
 	}
+
+	/**
+	 * {@inheritdoc}
+	 */
+	public function dispatch($eventName, Event $event = null)
+	{
+		if ($this->disabled)
+		{
+			return $event;
+		}
+
+		foreach ((array) $eventName as $name)
+		{
+			$event = parent::dispatch($name, $event);
+		}
+
+		return $event;
+	}
+
+	/**
+	 * {@inheritdoc}
+	 */
+	public function disable()
+	{
+		$this->disabled = true;
+	}
+
+	/**
+	 * {@inheritdoc}
+	 */
+	public function enable()
+	{
+		$this->disabled = false;
+	}
 }
