@@ -16,20 +16,37 @@ namespace phpbb\event;
 use Symfony\Component\EventDispatcher\EventSubscriberInterface;
 use Symfony\Component\HttpFoundation\JsonResponse;
 use Symfony\Component\HttpKernel\Exception\HttpExceptionInterface;
+use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
 use Symfony\Component\HttpKernel\KernelEvents;
 use Symfony\Component\HttpKernel\Event\GetResponseForExceptionEvent;
 use Symfony\Component\HttpFoundation\Response;
 
 class kernel_exception_subscriber implements EventSubscriberInterface
 {
+	/**
+	 * Set to true to show full exception messages
+	 *
+	 * @var bool
+	 */
+	protected $debug;
+
 	/**
 	* Template object
+	*
 	* @var \phpbb\template\template
 	*/
 	protected $template;
 
+	/**
+	* Language object
+	*
+	* @var \phpbb\language\language
+	*/
+	protected $language;
+
 	/**
 	* User object
+	*
 	* @var \phpbb\user
 	*/
 	protected $user;
@@ -40,12 +57,16 @@ class kernel_exception_subscriber implements EventSubscriberInterface
 	/**
 	* Construct method
 	*
-	* @param \phpbb\template\template $template Template object
-	* @param \phpbb\user $user User object
+	* @param \phpbb\template\template	$template	Template object
+	* @param \phpbb\language\language	$language	Language object
+	* @param \phpbb\user				$user		User object
+	* @param bool						$debug		Set to true to show full exception messages
 	*/
-	public function __construct(\phpbb\template\template $template, \phpbb\user $user)
+	public function __construct(\phpbb\template\template $template, \phpbb\language\language $language, \phpbb\user $user, $debug = false)
 	{
+		$this->debug = $debug || defined('DEBUG');
 		$this->template = $template;
+		$this->language = $language;
 		$this->user = $user;
 		$this->type_caster = new \phpbb\request\type_cast_helper();
 	}
@@ -61,19 +82,32 @@ class kernel_exception_subscriber implements EventSubscriberInterface
 		$exception = $event->getException();
 
 		$message = $exception->getMessage();
-		$this->type_caster->set_var($message, $message, 'string', false, false);
+		$this->type_caster->set_var($message, $message, 'string', true, false);
 
 		if ($exception instanceof \phpbb\exception\exception_interface)
 		{
-			$message = call_user_func_array(array($this->user, 'lang'), array_merge(array($message), $exception->get_parameters()));
+			$message = $this->language->lang_array($message, $exception->get_parameters());
 		}
+		else if (!$this->debug && $exception instanceof NotFoundHttpException)
+		{
+			$message = $this->language->lang('PAGE_NOT_FOUND');
+		}
+
+		// Do not update user session page if it does not exist
+		if ($exception instanceof NotFoundHttpException)
+		{
+			$this->user->update_session_page = false;
+		}
+
+		// Show <strong> text in bold
+		$message = preg_replace('#&lt;(/?strong)&gt;#i', '<$1>', $message);
 
 		if (!$event->getRequest()->isXmlHttpRequest())
 		{
-			page_header($this->user->lang('INFORMATION'));
+			page_header($this->language->lang('INFORMATION'));
 
 			$this->template->assign_vars(array(
-				'MESSAGE_TITLE' => $this->user->lang('INFORMATION'),
+				'MESSAGE_TITLE' => $this->language->lang('INFORMATION'),
 				'MESSAGE_TEXT'  => $message,
 			));
 
@@ -94,7 +128,7 @@ class kernel_exception_subscriber implements EventSubscriberInterface
 				$data['message'] = $message;
 			}
 
-			if (defined('DEBUG'))
+			if ($this->debug)
 			{
 				$data['trace'] = $exception->getTrace();
 			}
@@ -111,7 +145,7 @@ class kernel_exception_subscriber implements EventSubscriberInterface
 		$event->setResponse($response);
 	}
 
-	public static function getSubscribedEvents()
+	static public function getSubscribedEvents()
 	{
 		return array(
 			KernelEvents::EXCEPTION		=> 'on_kernel_exception',
