@@ -25,14 +25,16 @@ class kernel_terminate_subscriber implements EventSubscriberInterface
 	* primarily cleanup stuff.
 	*
 	* @param PostResponseEvent $event
-	* @return null
+	* @return void
 	*/
 	public function on_kernel_terminate(PostResponseEvent $event)
 	{
+		garbage_collection();
+
 		exit_handler();
 	}
 
-	public static function getSubscribedEvents()
+	static public function getSubscribedEvents()
 	{
 		return array(
 			KernelEvents::TERMINATE		=> array('on_kernel_terminate', ~PHP_INT_MAX),
