@@ -168,7 +168,7 @@ class quote extends \phpbb\notification\type\post
 		$user_data = $this->user_loader->get_user($this->get_data('poster_id'));
 
 		return array_merge(parent::get_email_template_variables(), array(
-			'AUTHOR_NAME'		=> htmlspecialchars_decode($user_data['username'], ENT_COMPAT),
+			'AUTHOR_NAME'		=> html_entity_decode($user_data['username'], ENT_COMPAT),
 		));
 	}
 
