@@ -87,7 +87,7 @@ class context
 	}
 
 	/**
-	* Retreive a single scalar value from a single key.
+	* Retrieve a single scalar value from a single key.
 	*
 	* @param string $varname Variable name
 	* @return mixed Variable value, or null if not set
@@ -149,7 +149,7 @@ class context
 			{
 				// If the key name is lowercase and the data is an array,
 				// it could be a template loop. So we set the S_NUM_ROWS there
-				// aswell.
+				// as well.
 				if ($sub_block_name === strtolower($sub_block_name) && is_array($sub_block))
 				{
 					$this->set_num_rows($sub_block);
@@ -201,7 +201,8 @@ class context
 			$pos = strpos($blocks[$i], '[');
 			$name = ($pos !== false) ? substr($blocks[$i], 0, $pos) : $blocks[$i];
 			$block = &$block[$name];
-			$index = (!$pos || strpos($blocks[$i], '[]') === $pos) ? (count($block) - 1) : (min((int) substr($blocks[$i], $pos + 1, -1), count($block) - 1));
+			$block_count = empty($block) ? 0 : count($block) - 1;
+			$index = (!$pos || strpos($blocks[$i], '[]') === $pos) ? $block_count : (min((int) substr($blocks[$i], $pos + 1, -1), $block_count));
 			$block = &$block[$index];
 		}
 
@@ -274,6 +275,11 @@ class context
 			{
 				$name = substr($blocks[$i], 0, $pos);
 
+				if (empty($block[$name]))
+				{
+					return array();
+				}
+
 				if (strpos($blocks[$i], '[]') === $pos)
 				{
 					$index = count($block[$name]) - 1;
@@ -286,6 +292,11 @@ class context
 			else
 			{
 				$name = $blocks[$i];
+				if (empty($block[$name]))
+				{
+					return array();
+				}
+
 				$index = count($block[$name]) - 1;
 			}
 			$block = $block[$name];
@@ -373,7 +384,9 @@ class context
 		if (is_array($key))
 		{
 			// Search array to get correct position
-			list($search_key, $search_value) = @each($key);
+			$search_key = key($key);
+			$search_value = current($key);
+
 			foreach ($block as $i => $val_ary)
 			{
 				if ($val_ary[$search_key] === $search_value)
@@ -470,7 +483,8 @@ class context
 		if (is_array($key))
 		{
 			// Search array to get correct position
-			list($search_key, $search_value) = @each($key);
+			$search_key = key($key);
+			$search_value = current($key);
 
 			$key = null;
 			foreach ($block as $i => $val_ary)
