@@ -15,7 +15,7 @@ namespace phpbb\template\twig;
 
 use phpbb\template\assets_bag;
 
-class environment extends \Twig_Environment
+class environment extends \Twig\Environment
 {
 	/** @var \phpbb\config\config */
 	protected $phpbb_config;
@@ -55,11 +55,11 @@ class environment extends \Twig_Environment
 	* @param \phpbb\path_helper $path_helper phpBB path helper
 	* @param string $cache_path The path to the cache directory
 	* @param \phpbb\extension\manager $extension_manager phpBB extension manager
-	* @param \Twig_LoaderInterface $loader Twig loader interface
+	* @param \Twig\Loader\LoaderInterface $loader Twig loader interface
 	* @param \phpbb\event\dispatcher_interface	$phpbb_dispatcher	Event dispatcher object
 	* @param array $options Array of options to pass to Twig
 	*/
-	public function __construct(\phpbb\config\config $phpbb_config, \phpbb\filesystem\filesystem $filesystem, \phpbb\path_helper $path_helper, $cache_path, \phpbb\extension\manager $extension_manager = null, \Twig_LoaderInterface $loader = null, \phpbb\event\dispatcher_interface $phpbb_dispatcher = null, $options = array())
+	public function __construct(\phpbb\config\config $phpbb_config, \phpbb\filesystem\filesystem $filesystem, \phpbb\path_helper $path_helper, $cache_path, \phpbb\extension\manager $extension_manager = null, \Twig\Loader\LoaderInterface $loader = null, \phpbb\event\dispatcher_interface $phpbb_dispatcher = null, $options = array())
 	{
 		$this->phpbb_config = $phpbb_config;
 
@@ -69,7 +69,6 @@ class environment extends \Twig_Environment
 		$this->phpbb_dispatcher = $phpbb_dispatcher;
 
 		$this->phpbb_root_path = $this->phpbb_path_helper->get_phpbb_root_path();
-		$this->web_root_path = $this->phpbb_path_helper->get_web_root_path();
 
 		$this->assets_bag = new assets_bag();
 
@@ -132,7 +131,7 @@ class environment extends \Twig_Environment
 	*/
 	public function get_web_root_path()
 	{
-		return $this->web_root_path;
+		return $this->web_root_path ?? $this->web_root_path = $this->phpbb_path_helper->get_web_root_path();
 	}
 
 	/**
@@ -169,7 +168,7 @@ class environment extends \Twig_Environment
 	* Set the namespace look up order to load templates from
 	*
 	* @param array $namespace
-	* @return \Twig_Environment
+	* @return \Twig\Environment
 	*/
 	public function setNamespaceLookUpOrder($namespace)
 	{
@@ -261,8 +260,8 @@ class environment extends \Twig_Environment
 	*
 	* @param string  $name  The template name
 	* @param integer $index The index if it is an embedded template
-	* @return \Twig_TemplateInterface A template instance representing the given template name
-	* @throws \Twig_Error_Loader
+	* @return \Twig\Template A template instance representing the given template name
+	* @throws \Twig\Error\LoaderError
 	*/
 	public function loadTemplate($name, $index = null)
 	{
@@ -279,7 +278,7 @@ class environment extends \Twig_Environment
 
 					return parent::loadTemplate('@' . $namespace . '/' . $name, $index);
 				}
-				catch (\Twig_Error_Loader $e)
+				catch (\Twig\Error\LoaderError $e)
 				{
 				}
 			}
@@ -298,7 +297,7 @@ class environment extends \Twig_Environment
 	*
 	* @param string  $name  The template name
 	* @return string
-	* @throws \Twig_Error_Loader
+	* @throws \Twig\Error\LoaderError
 	*/
 	public function findTemplate($name)
 	{
@@ -315,7 +314,7 @@ class environment extends \Twig_Environment
 
 					return parent::getLoader()->getCacheKey('@' . $namespace . '/' . $name);
 				}
-				catch (\Twig_Error_Loader $e)
+				catch (\Twig\Error\LoaderError $e)
 				{
 				}
 			}
