@@ -13,15 +13,14 @@
 
 namespace phpbb\template\twig\node;
 
-
-class includenode extends \Twig_Node_Include
+class includenode extends \Twig\Node\IncludeNode
 {
 	/**
 	* Compiles the node to PHP.
 	*
-	* @param \Twig_Compiler A Twig_Compiler instance
+	* @param \Twig\Compiler A Twig\Compiler instance
 	*/
-	public function compile(\Twig_Compiler $compiler)
+	public function compile(\Twig\Compiler $compiler)
 	{
 		$compiler->addDebugInfo($this);
 
