@@ -26,7 +26,7 @@ phpbb.addAjaxCallback('mark_forums_read', function(res) {
 	});
 
 	// Mark subforums read
-	$('a.subforum[class*="unread"]').removeClass('unread').addClass('read');
+	$('a.subforum[class*="unread"]').removeClass('unread').addClass('read').children('.icon.icon-red').removeClass('icon-red').addClass('icon-blue');
 
 	// Mark topics read if we are watching a category and showing active topics
 	if ($('#active_topics').length) {
@@ -87,7 +87,7 @@ phpbb.addAjaxCallback('mark_topics_read', function(res, updateTopicLinks) {
 	});
 
 	// Remove link to first unread post
-	$('a').has('span.icon_topic_newest').remove();
+	$('a.unread').has('.icon-red').remove();
 
 	// Update mark topics read links
 	if (updateTopicLinks) {
@@ -101,6 +101,7 @@ phpbb.addAjaxCallback('mark_topics_read', function(res, updateTopicLinks) {
 phpbb.addAjaxCallback('notification.mark_all_read', function(res) {
 	if (typeof res.success !== 'undefined') {
 		phpbb.markNotifications($('#notification_list li.bg2'), 0);
+		phpbb.toggleDropdown.call($('#notification_list_button'));
 		phpbb.closeDarkenWrapper(3000);
 	}
 });
@@ -132,9 +133,10 @@ phpbb.markNotifications = function($popup, unreadCount) {
 
 	// Update the unread count.
 	$('strong', '#notification_list_button').html(unreadCount);
-	// Remove the Mark all read link if there are no unread notifications.
+	// Remove the Mark all read link and hide notification count if there are no unread notifications.
 	if (!unreadCount) {
 		$('#mark_all_notifications').remove();
+		$('#notification_list_button > strong').addClass('hidden');
 	}
 
 	// Update page title
@@ -198,7 +200,7 @@ phpbb.addAjaxCallback('zebra', function(res) {
  */
 phpbb.addAjaxCallback('vote_poll', function(res) {
 	if (typeof res.success !== 'undefined') {
-		var poll = $('.topic_poll');
+		var poll = $(this).closest('.topic_poll');
 		var panel = poll.find('.panel');
 		var resultsVisible = poll.find('dl:first-child .resultbar').is(':visible');
 		var mostVotes = 0;
@@ -335,7 +337,6 @@ $('[data-ajax]').each(function() {
 	}
 });
 
-
 /**
  * This simply appends #preview to the action of the
  * QR action when you click the Full Editor & Preview button
@@ -360,6 +361,17 @@ $('.display_post').click(function(e) {
 	$('#post_hidden' + postId).hide();
 });
 
+/**
+ * Display hidden post on post review page
+ */
+$('.display_post_review').on('click', function(e) {
+	e.preventDefault();
+
+	let $displayPostLink = $(this);
+	$displayPostLink.closest('.post-ignore').removeClass('post-ignore');
+	$displayPostLink.hide();
+});
+
 /**
 * Toggle the member search panel in memberlist.php.
 *
